//
// AMX Mod X SetAnimation Forward
// Copyright (C) Vincent Herbet (Arkshine)
//
// This software is licensed under the GNU General Public License, version 2.
// For full license details, see LICENSE file.
//

#include "setanimation.h"
#include "gamedatas.h"
#include "utils.h"

#include <MemoryUtils.h>
#include <CDetour/detours.h>

CDetour* SetAnimationDetour;

typedef void(*SetAnimationFn)(const void*, int);
SetAnimationFn SetAnimationOrig;

extern int OnSetAnimationForward;

DETOUR_DECL_MEMBER1(SetAnimation, void, int, playerAnim)
{
	const void *pvPlayer = reinterpret_cast<const void*>(this);

#if defined(LINUX)

	asm volatile
	(
		"movl %%edx, %0;"
		"movl %%eax, %1;"
		: "=d" (playerAnim), "=a" (pvPlayer) : :
	);

#endif

	if (MF_ExecuteForward(OnSetAnimationForward, Utils::PrivateToIndex(pvPlayer), playerAnim) > 0)
	{
		return;
	}

#if defined(WIN32) || defined(APPLE)

	DETOUR_MEMBER_CALL(SetAnimation)(playerAnim);

#elif defined(LINUX)

	SetAnimationDetour->DisableDetour();
	SetAnimationOrig(pvPlayer, playerAnim);
	SetAnimationDetour->EnableDetour();

#endif
}

void ActivateDetour()
{
	void* funcAddress = Utils::FindAddressFromEntry(FUNC_SETANIMATION);

#if defined(LINUX)
	SetAnimationOrig = reinterpret_cast<SetAnimationFn>(Utils::FindAddressFromEntry(FUNC_SETANIMATION_ORIG));
#endif

	SetAnimationDetour = DETOUR_CREATE_MEMBER_FIXED(SetAnimation, funcAddress);

#if defined(LINUX)
	if (SetAnimationDetour && funcAddress && SetAnimationOrig)
#else
	if (SetAnimationDetour && funcAddress)
#endif
	{
		SetAnimationDetour->EnableDetour();
	}
	else
	{
		if (!funcAddress)
		{
			SERVER_PRINT("[SETANIMATION] CBasePlayer::SetAnimation (part2) address could not be found.\n");
		}
#if defined(LINUX)

		if (!SetAnimationOrig)
		{
			SERVER_PRINT("[SETANIMATION] CBasePlayer::SetAnimation (original) address could not be found.\n");
		}
#endif
	}
}

void DestroyDetour()
{
	if (SetAnimationDetour)
	{
		SetAnimationDetour->Destroy();
	}
}
